function [V, H,  Vd] = get_welfare(orig_eta, fout, Dyn, s)
% Calculate welfare using a finite difference scheme.
% Outputs are
% 1. V   (total welfare)
% 2. H   (defined in text)
% 3. Vc  (only flow consumption utility)
% 4. Vd  (only flow convenience yield on deposits)
% 5. Vpi (only flow inflation costs)
%
% Written by William Chen and Greg Phelan, Jan. 2022
% GREG KILLING ALL THE NON-LIQUIDITY TERMS

orig_theta=fout(:,1);
orig_Q= s.ab./(s.r.*(1+orig_eta.*(orig_theta-1)));

%dyn = [psi, sigma_eta, sigma_Q, mu_eta, sigma_ch, omega, mu_theta, sigma_theta]

% Grab values
orig_psi             = Dyn(:, 1);
orig_sigma_eta       = Dyn(:, 2);
orig_mu_eta          = Dyn(:, 4);
orig_mu_theta        = Dyn(:, 7);
orig_sigma_theta     = Dyn(:, 8);
orig_theta           = fout(:, 1);
% orig_Q               = Q;
orig_eta_mu_eta      = orig_eta .* orig_mu_eta;
orig_eta_sigma_etasq = (orig_eta .* orig_sigma_eta).^2;

%I'm hard-wiring this to be total debt now
orig_x_hd            = (Dyn(:, 1) - orig_eta);% ./ (1 - orig_eta);

% Interpolate to uniform grid so the centered difference
% approximation is properly constructed
eta = linspace(orig_eta(1), orig_eta(end), s.N_welfare)';
psi             = interp1(orig_eta, orig_psi,          eta, s.interp);
mu_eta          = interp1(orig_eta, orig_mu_eta,          eta, s.interp);
sigma_eta       = interp1(orig_eta, orig_sigma_eta,       eta, s.interp);
mu_theta        = interp1(orig_eta, orig_mu_theta,        eta, s.interp);
sigma_theta     = interp1(orig_eta, orig_sigma_theta,     eta, s.interp);
theta           = interp1(orig_eta, orig_theta,           eta, s.interp);
Q               = interp1(orig_eta, orig_Q,               eta, s.interp);
eta_mu_eta      = interp1(orig_eta, orig_eta_mu_eta,      eta, s.interp);
eta_sigma_etasq = interp1(orig_eta, orig_eta_sigma_etasq, eta, s.interp);
x_hd            = interp1(orig_eta, orig_x_hd,            eta, s.interp);
% inflation       = interp1(orig_eta, orig_inflation,       eta, s.interp);

g_eta          =  psi.*s.g+(1-psi).*s.gh;
% Set up finite difference operators via upwind scheme
% with reflecting boundary conditions. At eta = 0,
% a reflecting boundary is fine b/c mu_eta > 0 at eta = 0
% and sigma_eta -> 0. With costly equity issuance,
% a reflecting boundary is the equilibrium outcome.
% At eta = etastar, the reflecting boundary
% is an equilibrium outcome.
dX = diff(eta);
N  = s.N_welfare;
L2coefs          = zeros(N, 1);
L2coefs(2:N - 1) = eta_sigma_etasq(2:N - 1) ./ (2 .* dX(1:N - 2) .* dX(2:N - 1));
L2coefs(end)     = eta_sigma_etasq(end) ./ (2 .* dX(end - 1) .* dX(end));
if s.issuance > 1 
    % then costly equity issuance => do not zero volatility out at boundary
    L2coefs(1)   = eta_sigma_etasq(1) ./ (2 .* dX(1) .* dX(2));
end

DU = zeros(N, 1);
DD = zeros(N, 1);
D0 = zeros(N, 1);

DU(2:N)     = max(eta_mu_eta(1:N - 1), 0) ./ dX + L2coefs(1:N - 1);
DD(1:N - 1) = max(-eta_mu_eta(2:N), 0.) ./ dX + L2coefs(2:N);
D0(1:N - 1) = -DU(2:N);
D0(2:N)     = D0(2:N) - DD(1:N - 1);
D0          = D0 - s.r .* ones(N, 1);

A = spdiags([DD D0 DU], -1:1, N, N);

% To check
% DU1 = zeros(N, 1); DU2 = zeros(N, 1);
% DD1 = zeros(N, 1); DD2 = zeros(N, 1);
% D01 = zeros(N, 1); D02 = zeros(N, 1);
% DU1(2:N) = max(eta_mu_eta(1:N - 1), 0) ./ dX;
% DU2(2:N) = L2coefs(1:N - 1);
% DD1(1:N - 1) = max(-eta_mu_eta(2:N), 0.) ./ dX;
% DD2(1:N - 1) = L2coefs(2:N);
% D01(1:N - 1) = -DU1(2:N);
% D02(1:N - 1) = -DU2(2:N);
% D01(2:N) = D01(2:N) - DD1(1:N - 1);
% D02(2:N) = D02(2:N) - DD2(1:N - 1);
% L1 = spdiags([DD1, D01, DU1], -1:1, N, N);
% L2 = spdiags([DD2, D02, DU2], -1:1, N, N);
% Acheck = L1 + L2 - spdiags(s.r * ones(N, 1), 0, N, N);

% Inhomogeneous parts
% muK          = s.g.*ones(size(theta));%(s.eps1 / s.eps2) .* log(s.eps1 .* Q) - s.delta;
% wealth_term1 = (theta .* eta .* (mu_theta + sigma_theta .* sigma_eta) + ...
%     (theta - 1) .* eta .* mu_eta) ./ (1 + (theta - 1) .* eta);
% wealth_term2 = (theta .* eta .* sigma_theta + (theta - 1) .* eta .* ...
%     sigma_eta).^2 ./ (2 .* (1 + (theta - 1) .* eta).^2);
% s.beta_1=adjustLiqPref(psi,eta,s);
conv_yield   = convenience_yield(x_hd.*Q*s.r, eta,0, s)+g_eta;
% pi_costs     =  0;%s.lambda / 2 * (inflation - s.pi_target).^2;
% Inhom        = log(s.r * Q) + conv_yield  ...
%     + (muK - s.sigma^2 / 2) / s.r + (wealth_term1 - wealth_term2) / s.r;

% Solve linear equation system to get welfare
% e.g. for total welfare,
% ((eta_sigma_eta)^2 / 2) * L2 * H + eta_mu_eta * L1 * H - r * H = -Inhom
% unif_H   = A \ (-Inhom);
unif_Vd  = A \ (-conv_yield);
% unif_Vpi = A \ (-pi_costs);

% Interpolate back to non-uniform grid
% and note that
% V = log(1 + (theta - 1) * eta) / r + H = Vc + Vd - Vpi;
% H   = interp1(eta, unif_H,   orig_eta);
Vd  = interp1(eta, unif_Vd,  orig_eta);
H=Vd;
% Vpi = interp1(eta, unif_Vpi, orig_eta);
% V   = log(1 + (orig_theta - 1) .* orig_eta) ./ s.r + H;
V   = log(s.ab)./s.r+Vd+(0-s.sigma^2/2)/s.r;
% Vc  = V - (Vd - Vpi);
end
