% This script generates results of robustness when
% bank intermediation raises the productivity of the tree
%
% Written by William Chen, April 2020

clear; clc; close all;
addpath parameter_specs
addpath eqm_functions

% What to run?
do_plots   = 1;         % are we making plots?
save_plots = 1;         % are we saving plots?
vary_a_    = 1;         % varying a_
vary_g_    = 1;         % varying g_
color_vec  = {[1, 0.5, 0], 'red', [.5, 0, .7], 'blue'};

% Initialize parameters
spec2;
s.display_time = 0;
if vary_a_
    gpath_a_ = strcat(s.gpath, 'banks_raise_productivity/spec2/vary_a_/');
end
if vary_g_
    gpath_g_ = strcat(s.gpath, 'banks_raise_productivity/spec2/vary_g_/');
end

% Pre allocate storage
fout_a_    = cell(length(vec_a_), 1); eta_a_ = cell(length(vec_a_), 1);
Dyn_a_     = cell(length(vec_a_), 1); pdf_a_ = cell(length(vec_a_), 1);
pdf_eta_a_ = cell(length(vec_a_), 1); cdf_a_ = cell(length(vec_a_), 1);
s_a_       = cell(length(vec_a_), 1);

fout_g_    = cell(length(vec_g_), 1); eta_g_ = cell(length(vec_g_), 1);
Dyn_g_     = cell(length(vec_g_), 1); pdf_g_ = cell(length(vec_g_), 1);
pdf_eta_g_ = cell(length(vec_g_), 1); cdf_g_ = cell(length(vec_g_), 1);
s_g_       = cell(length(vec_g_), 1);

% Solve model for multiple levels of productivity drops
if vary_a_
    disp('Solving the model varying a_');
    a_time = tic;
    for i = 1:length(vec_a_)
        s.a_ = vec_a_(i);
        [fout_a_{i}, eta_a_{i}, Dyn_a_{i}, pdf_a_{i}, pdf_eta_a_{i}, cdf_a_{i}, s_a_{i}] = ...
            GetEqmIssuanceNoPlots(s, 0);
    end
    toc(a_time);
end

if vary_g_
    disp('Solving the model varying g_');
    g_time = tic;
    for i = 1:length(vec_g_)
        s.g_ = vec_g_(i);
        [fout_g_{i}, eta_g_{i}, Dyn_g_{i}, pdf_g_{i}, pdf_eta_g_{i}, cdf_g_{i}, s_g_{i}] = ...
            GetEqmIssuanceNoPlots(s, 0);
    end
    toc(g_time);
end

% Plot results!
if do_plots
    if vary_a_
        disp('Plotting results for a_ . . .');

        legend_labels = cell(length(vec_a_), 1);
        for i = 1:length(vec_a_)
            fout    = fout_a_{i};    eta = eta_a_{i};
            Dyn     = Dyn_a_{i};     pdf = pdf_a_{i};
            pdf_eta = pdf_eta_a_{i}; cdf = cdf_a_{i};
            s       = s_a_{i};
            s.color = color_vec{i};
            default_plots;
            legend_labels{i} = strcat('$\underline{a} = $', num2str(vec_a_(i)));
        end
        for i = 1:11
            figure(i);
            legend(legend_labels, 'Interpreter', 'latex', 'FontSize', s.fontsize); hold off
        end
        if save_plots
            if ~isfolder(gpath_a_)
                mkdir(gpath_a_)
            end
            disp('Saving results for a_ . . .');
            saveas(figure(1), [gpath_a_, 'welfare.pdf']);
            saveas(figure(2), [gpath_a_, 'flow_utility.pdf']);
            saveas(figure(3), [gpath_a_, 'stationary_density.pdf']);
            saveas(figure(4), [gpath_a_, 'psi.pdf']);
            saveas(figure(7), [gpath_a_, 'sigma_eta_eta.pdf']);
            saveas(figure(8), [gpath_a_, 'mu_eta_eta.pdf']);
            saveas(figure(9), [gpath_a_, 'stationary_density_normalize.pdf']);
            saveas(figure(10), [gpath_a_, 'sharpe_ratio.pdf']);
        end
    end

    if vary_g_
        disp('Plotting results for g_ . . .');

        legend_labels = cell(length(vec_g_), 1);
        for i = 1:length(vec_g_)
            fout    = fout_g_{i};    eta = eta_g_{i};
            Dyn     = Dyn_g_{i};     pdf = pdf_g_{i};
            pdf_eta = pdf_eta_g_{i}; cdf = cdf_g_{i};
            s       = s_g_{i};
            s.color = color_vec{i};
            default_plots;
            legend_labels{i} = strcat('\underline{g} = ', num2str(vec_g_(i)));
        end
        for i = 1:11
            figure(i);
            legend(legend_labels, 'Interpreter', 'latex', 'FontSize', s.fontsize); hold off
        end
        if save_plots
            if ~isfolder(gpath_g_)
                mkdir(gpath_g_)
            end
            
            disp('Saving results for g_ . . .')
            saveas(figure(1), [gpath_g_, 'welfare.pdf']);
            saveas(figure(2), [gpath_g_, 'flow_utility.pdf']);
            saveas(figure(3), [gpath_g_, 'stationary_density.pdf']);
            saveas(figure(4), [gpath_g_, 'psi.pdf']);
            saveas(figure(5), [gpath_g_, 'q.pdf']);
            saveas(figure(7), [gpath_g_, 'sigma_eta_eta.pdf']);
            saveas(figure(8), [gpath_g_, 'mu_eta_eta.pdf']);
            saveas(figure(9), [gpath_g_, 'stationary_density_normalize.pdf']);
            saveas(figure(10), [gpath_g_, 'sharpe_ratio.pdf']);
        end
    end
end
