function [fp, dyn, risk_premium, Sharpe] = fnct(eta, f, s)
% [fp dyn] = fnct(eta, f, s)
% takes scalar eta, a 4x1 vector f = [theta, theta', q, q'], and parameters
% r, rho, a, sigma, g, delta_, and computes fp, the derivative of f with
% respect to eta, as well as dyn = [psi, sigma_eta*eta, sigma_q, mu_eta*eta, mu_q]
% 
% Allows fire sales from banks to households.
%
% written by Greg Phelan and William Chen
%GREG ADDED DEPO_SPREAD FUNCTION

theta=f(1);
%% Figure out bounds from leverage constraint
if ~isnumeric(s.K)
    K = (1 + eta * f(4) / f(3)) / (s.sigma + eta * f(4) / f(3));
    bound = K * eta * 1;
else
    if s.K == 0
        bound = 2;
    else
        bound = s.K * eta;
    end
end

%% Solve for psi
tol = s.psi_tol;
psi_L = eta; psi_R = min([abs(f(3) / f(4)) + eta, 1 + tol, bound + s.bound_tol]);  % alpha
for n = 1:s.N_bisection_iter
    psi = (psi_L + psi_R)/2;
    amplification = 1 - f(4)/f(3)*(psi - eta);

    % VOLATILITY COMPUTATION            
    sigma_eta_eta = s.sigma*(psi - eta)/amplification;  % sigma_eta *times* eta            
    sigma_q = sigma_eta_eta*f(4)/f(3); 
    sigma_theta = sigma_eta_eta*f(2)/f(1);

    %calculate the deposit spread
    phiL=depo_spread(psi,eta,theta,s);
    
    % banks FOC => a / q + g  + mu_q + sigma * sigma_q - (r - phiL) = risk_premium
    % HH FOC    => a_   / q + g_ + mu_q + sigma * sigma_q - r          = 0
    risk_premium = (- sigma_theta)*(s.sigma + sigma_q);
    household_premium = (s.a_ - s.a) / f(3) + (s.g_ - s.g) - phiL + risk_premium;
    
    if household_premium > 0 % households want to hold more
        psi_R = psi;
    else
        psi_L = psi;
    end
end
psi = min([(psi_R + psi_L) / 2, 1, bound]);

%% VOLATILITY COMPUTATION 
amplification = 1 - f(4) /f(3) * (psi - eta);      
sigma_eta_eta = s.sigma * (psi - eta) / amplification;             
sigma_q       = sigma_eta_eta * f(4) / f(3); 
sigma_theta   = sigma_eta_eta * f(2) / f(1);
risk_premium  = -sigma_theta * (s.sigma + sigma_q);
Sharpe        = -sigma_theta;

%% Calculate drifts for ODEs
if psi < bound % Unconstrained
    mu_q       = s.r - phiL - s.a / f(3) - s.g - s.sigma * sigma_q + risk_premium;
    mu_theta   = phiL;
    mu_eta_eta = (psi - eta) * (risk_premium - (s.sigma + sigma_q)^2) + ...
            eta * (s.a / f(3) + (1 - psi) * (s.g_ - s.g));
else % Leverage constraint binds
    mu_q       = s.r - s.a_ / f(3) - s.g_ - s.sigma * sigma_q;   
    mu_a       = s.a / f(3) + s.g + mu_q + s.sigma*sigma_q;   
    mu_theta   = max(phiL - psi / eta * ...
            (mu_a - s.r + sigma_theta * (s.sigma + sigma_q)), -50); %put a bound to catch numerical errors
    mu_eta_eta = psi * s.a / f(3) - (psi - eta) * (s.a_ / f(3) - phiL) + ...
            psi * (1 - eta) * (s.g - s.g_) - (psi - eta) * (s.sigma + sigma_q)^2;
end

qpp = 2*(mu_q*f(3) - f(4)*mu_eta_eta)/sigma_eta_eta^2; 
thetapp = 2*(mu_theta*f(1) - f(2)*mu_eta_eta)/sigma_eta_eta^2; 

fp = [f(2); thetapp; f(4); qpp];

% expert_premium = risk_premium;
% Sharpe=(mu_a-s.r)/(s.sigma + sigma_q);

dyn = [psi, sigma_eta_eta, sigma_q, mu_eta_eta, mu_q, mu_theta, Sharpe, risk_premium];


end
    


